# source('tauplot.R'); r = tauplot(1,2,4,1)
library(rgl)    # base 3d graphics library 
library(misc3d) # miscelaneous 3d graphics library 

ball = function(x,y,z){ sqrt(x^2+y^2+z^2) }
tau = function (x,y,z,sig1=1,sig2=2,sig3=3){ 
    ball( (sig1-sig2)*x*y, (sig1-sig3)*x*z, (sig2-sig3)*y*z ) }

plotUnitBall = function(ballc='blue'){    
    x = seq(-2,2,len=200)
    contour3d(ball,1,x,x,x,color=ballc,alpha=1,add=T) }
plotTauIsConstant = function(sig1=1,sig2=2,sig3=3,tauc='red',isp=1){    
    tausp = abs(c(sig3,sig1,sig2)-c(sig2,sig3,sig1))/2
    f = function(x,y,z){ tau(x,y,z,sig1,sig2,sig3) }
    x = seq(-2,2,len=200)
    contour3d(f,tausp[isp],x,x,x,color=tauc,alpha=1,add=T) }
plotStationaryPoints = function(spat){     
    plot3d(spat$x,spat$y,spat$z,spat$col,size=2,type='s',add=T) }
plotStationaryArrows = function(spat){     
    d = 1+(0:100/100)
    for (i in 1:4){
        plot3d(spat$x[i]*d,spat$y[i]*d,spat$z[i]*d,spat$col,size=1,type='s',add=T)
    } }
plotStationaryText = function(spat){     
    for (i in 1:4){
        text3d(spat$x[i]*2.1,spat$y[i]*2.1,spat$z[i]*2.1,spat$t[i],add=T)
    } }
plotCoordinateSystem = function(coor){     
    d = 1.5+(0:100/100)
    for (i in 1:3){
        plot3d(coor$x[i]*d,coor$y[i]*d,coor$z[i]*d,coor$col[i],size=0.6,type='s',add=T)
        text3d(coor$x[i]*2.6,coor$y[i]*2.6,coor$z[i]*2.6,coor$t[i],add=T)
    } }

tauplot = function(sig1=1,sig2=2,sig3=3,isp=1){
    spc = list('green', 'blue', 'yellow') # stationary points color
    spt = list('I', 'II', 'III')          # stationary points numbering
# plot of the unit ball 
    pball = plotUnitBall('blue')
# plot of the surface tau = const
    ptau = plotTauIsConstant(sig1,sig2,sig3,'red',isp)
# stationary points
    s = sqrt(2)/2; rep4 = function(x) { rep(x, times=4) }
    sp1 = data.frame(x=rep4(0),y=c(s,-s,s,-s),z=c(s,s,-s,-s),col=rep4(1),t=rep4('I'))    # stationary points for isp=1
    sp2 = data.frame(y=rep4(0),x=c(s,-s,s,-s),z=c(s,s,-s,-s),col=rep4(2),t=rep4('II'))   # stationary points for isp=2
    sp3 = data.frame(z=rep4(0),x=c(s,-s,s,-s),y=c(s,s,-s,-s),col=rep4(3),t=rep4('III'))  # stationary points for isp=3
    spat = list(sp1,sp2,sp3) # all stationary points
# stationary points annotation
    pspat = plotStationaryPoints(spat[[isp]])
    pspt =  plotStationaryText(spat[[isp]]) 
    pspa =  plotStationaryArrows(spat[[isp]])
# plot Cartesian coordinate system with basis e1, e2 and e3
    coor = data.frame(x=c(1,0,0),y=c(0,1,0),z=c(0,0,1),col=c(4,4,4),t=c('1','2','3'))   
    pcoor = plotCoordinateSystem(coor) }










#plotStationaryArrowsOld = function(sp){
#    p1 = matrix(c(sp$x,sp$y,sp$z),nrow=4)
#    p2 = p1*2    
#    t = sp$t
##    axes3d(t)
#    for (i in 1:4){
#        axis3d('x',pos=p2[i,],at=p1[i,],add=T) 
#    }
#}

#plotStationaryText = function(t,p){    
#    text3d(t,p*1.05, fontsize=80, color='black') 
#}
#plotCoordinateSystem = function(CS,c){
#    axes3d(c('1','2','3'))
#     COOR = matrix(c(1,0,0,0,1,0,0,0,1),nrow=3)
#    (sum(arrow((0,0,0),CS[i], rgbcolor=c, width=2, arrowsize=5) for i in [0..2]) + \
#            sum(text3d(i+1,CS[i]*1.05,fontsize=12,color=c) for i in [0..2])) 
#}



## Show 12 random vertices in various ways. 

#M = matrix(rnorm(36), 3, 12, dimnames=list(c('x','y','z'), 
#                                       rep(LETTERS[1:4], 3)))

## Force 4-tuples to be convex in planes so that quads3d works.

#for (i in c(1,5,9)) {
#    quad = as.data.frame(M[,i+0:3])
#    coeffs = runif(2,0,3)
#    if (mean(coeffs) < 1) coeffs = coeffs + 1 - mean(coeffs)
#    quad$C = with(quad, coeffs[1]*(B-A) + coeffs[2]*(D-A) + A)
#    M[,i+0:3] = as.matrix(quad)
#}

#open3d()

## Rows of M are x, y, z coords; transpose to plot

#M = t(M)
#shift = matrix(c(-3,3,0), 12, 3, byrow=TRUE)

#points3d(M)
#lines3d(M + shift)
#segments3d(M + 2*shift)
#triangles3d(M + 3*shift, col='red')
#quads3d(M + 4*shift, col='green')  
#text3d(M + 5*shift, texts=1:12)

## Add labels

#shift = outer(0:5, shift[1,])
#shift[,1] = shift[,1] + 3
#text3d(shift, 
#       texts = c('points3d','lines3d','segments3d',
#         'triangles3d', 'quads3d','text3d'),
#       adj = 0)
# rgl.bringtotop()


##Example 1: Draw a ball
#f = function(x, y, z)x^2+y^2+z^2
#x = seq(-2,2,len=20)
#contour3d(f,4,x,x,x)
#contour3d(f,4,x,x,x, engine = 'standard')
## ball with one corner removed.
#contour3d(f,4,x,x,x, mask = function(x,y,z) x > 0 | y > 0 | z > 0)
#contour3d(f,4,x,x,x, mask = function(x,y,z) x > 0 | y > 0 | z > 0,
#engine='standard', screen = list(x = 290, y = -20),
#color = 'red', color2 = 'white')
## ball with computed colors
#w = function(x,y,z) {
#v = sin(x) + cos(2 * y) * sin(5 * z)
#r = range(v)
#n = 100
#i = pmax(pmin(ceiling(n * (v - r[1]) / (r[2] - r[1])), n), 1)
#terrain.colors(n)[i]
#}
#contour3d(f,4,x,x,x, color = w)


## tests
#P = [tauplot(1,2,4,i) for i in [0..0]]
#h = sum(P[i].translate((8*i,0,0)) for i in range(len(P)))
#h.show(viewer='jmol', frame=false, figsize=[10,10], aspect_ratio=1)

