function is =  tauplot(sig11, sig22, sig33, isp)
% call: tauplot(sig11, sig22, sig33, isp)
% function plots the surface tau = taumax with the unit ball (inserted) 
% input parameters: sig11, sig22, sig33 are the principal stresses
%                   isp is the index of the plot group of the stationary points
    if nargin < 3, sig=[1 2 3];	else sig = [sig11 sig22 sig33];	end
    if nargin < 4, isp=1; end
    R = @(x,y,z) sqrt(x.^2 + y.^2 + z.^2); % ball
    tau = @(x,y,z, sig) R((sig11-sig22)*x.*y, (sig11-sig33)*x.*z, (sig22-sig33)*y.*z); % tau_{xi eta}
% stationary (extreme) shear stresses 
    tausp = abs(sig([2 3 1]) - sig([3 1 2]))/2; % stationary shear stresses
    taumax = max(abs(tausp));  % max shear stress
    if taumax == 0, error('Hidrostatic stress state.'); end
% domain plot points
    xmax = 2; ymax = 2; zmax = 2; d = -1:0.05:1;
    x = d*xmax; y = d*ymax; z = d*zmax;
    [X,Y,Z] = meshgrid(x,y,z);
% open figure
    figure(1); clf; hold on;
% plot of the principal axes 1 2 3
    Xmax = xmax+1; Ymax = ymax+1; Zmax = zmax+1; 
    z2 = [0 0];
    e1 = line([0 Xmax],z2,z2); e2 = line(z2,[0 Ymax],z2); e3 = line(z2,z2,[0 Zmax]);
    set([e1 e2 e3],'Color','k','LineWidth',1);
%   the principal axes annotations
    t = text([Xmax 0 0],[0 Ymax 0],[0 0 Zmax],{'1' '2' '3'},'FontSize',18,'Interpreter','tex'); 
% plot of the unit ball
    r = R(X,Y,Z);
    is = isosurface(X,Y,Z,r,1,r);
    p = patch(is); 
    isonormals(X,Y,Z,r,p); set(p,'FaceColor','b','EdgeColor','none');
    camlight; lighting gouraud;
% plot of the surface tau(x1,y1,z1) = const
    T = tau(X,Y,Z, sig);
    p = patch(isosurface(X,Y,Z,T,tausp(isp),r)); isonormals(X,Y,Z,T,p); set(p,'FaceColor','r','EdgeColor','none');
    camlight; lighting gouraud;
% plot of the gradients at the shear stress stationary points 
    sp1 = [0 -1 -1; 0 -1 1; 0 1 -1; 0 1 1] * sqrt(2)/2; % stationary points for isp=1
    sp2 = [-1 0 -1; -1 0 1; 1 0 -1; 1 0 1] * sqrt(2)/2; % stationary points for isp=2
    sp3 = [-1 -1 0; -1 1 0; 1 -1 0; 1 1 0] * sqrt(2)/2; % stationary points for isp=3
    sp  = {sp1, sp2, sp3}; % all stationary points
%   the gradient annotations
    spisp = sp{isp}; % stationary points corresponding to index isp group
    xp    = spisp(:,1); yp = spisp(:,2); zp = spisp(:,3);
    tp    = {'I' 'II' 'III'}; % stationary point annotation
    for i = 1:4
        txt   = text(xp(i)*1.5, yp(i)*1.5, zp(i)*1.5, tp{isp}, 'FontSize',18,'Interpreter','tex');
        line(xp(i),yp(i),zp(i),'MarkerEdgeColor','g','MarkerFaceColor','g','Marker','o','MarkerSize',10);
    end
%   the gradient plot
    for i = 1:4, line([xp(i) 0]*1.5,[yp(i) 0]*1.5,[zp(i) 0]*1.5,'Color','g'); end
    axis equal; axis off;
end


% # geslo za Windows XP + SP3
% # CT3DR-TKJX3-TD4H9-QGGKJ-XWK3W 

%    spt = @(i,j) full(sparse([1 1 2 2 3 3 4 4],[i j i j i j i j],[-1 -1 -1 1 1 -1 1 1]*sqrt(2)/2,4,3));
%    sp = {spt(2,3), spt(1,3), spt(1,2)}; % all stationary points

% 
%	[faces,verts,colors] = isosurface(X,Y,Z,T,tausp(isp),r);
%	patch('Vertices', verts, 'Faces', faces, 'FaceVertexCData', colors, 'FaceColor','interp', 'edgecolor', 'interp');
%    view(30,-15); axis vis3d; colormap copper
















